@@ -2,6 +2,57 @@
 
 CPAN::YACSmoke - Yet Another CPAN Smoke Tester
 
+=begin readme
+
+=head1 REQUIREMENTS
+
+This package requires the following modules (most of which are not
+included with Perl):
+
+  CPANPLUS
+  Config::IniFiles
+  File::Basename
+  File::HomeDir
+  File::Path
+  File::Spec
+  File::Temp
+  IO::File
+  LWP::Simple
+  Module::Pluggable
+  Path::Class
+  Regexp::Assemble
+  SDBM_File
+  Sort::Versions
+  Test::Reporter
+  URI
+  if
+
+These dependencies (such as L<CPANPLUS> and L<Test::Reporter>) may require
+additional modules.
+
+Windows users should also have L<File::HomeDir::Win32> installed.
+
+=head1 INSTALLATION
+
+Installation can be done using the traditional Makefile.PL or the newer
+Build.PL methods.
+
+Using Makefile.PL:
+
+  perl Makefile.PL
+  make test
+  make install
+
+(On Windows platforms you should use C<nmake> instead.)
+
+Using Build.PL (if you have Module::Build installed):
+
+  perl Build.PL
+  perl Build test
+  perl Build install
+
+=end readme
+
 =head1 SYNOPSIS
 
   perl -MCPAN::YACSmoke -e test
@@ -11,6 +62,18 @@ CPAN::YACSmoke - Yet Another CPAN Smoke Tester
 This module uses the backend of L<CPANPLUS> to run tests on modules
 recently uploaded to CPAN and post results to the CPAN Testers list.
 
+=begin readme
+
+See the module documentation for more information.
+
+=head1 REVISION HISTORY
+
+=for readme include file=Changes type=text start=0.03 stop=0.03_05
+
+=end readme
+
+=for readme stop
+
 It will create a database file in the F<.cpanplus> directory, which it
 uses to track tested distributions.  This information will be used to
 keep from posting multiple reports for the same module, and to keep
@@ -37,8 +100,9 @@ use CPANPLUS::Error;
 use File::Path;
 use File::Basename;
 use File::HomeDir qw( home );
-use File::Spec::Functions qw( splitpath catfile );
+use File::Spec::Functions qw( splitpath );
 use LWP::Simple;
+use Path::Class;
 use POSIX qw( O_CREAT O_RDWR );         # for SDBM_File
 use Regexp::Assemble;
 use SDBM_File;
@@ -48,12 +112,14 @@ use Module::Pluggable search_path => ["CPAN::YACSmoke::Plugin"];
 use Carp;
 use Config::IniFiles;
 
+use if ($^O eq "MSWin32"), "File::HomeDir::Win32";
+
 # use YAML 'Dump';
 
 require Test::Reporter;
 require YAML;
 
-our $VERSION = '0.03_05';
+our $VERSION = '0.03_07';
 $VERSION = eval $VERSION;
 
 require Exporter;
@@ -135,6 +201,7 @@ my $extn = qr/(?:\.(?:tar\.gz|tgz|zip))/;	# supported archive extensions
         code => sub {
 		  my $mod    = shift;
 		  my $report = shift || "";
+		  $report =~ s/\[MSG\] \[[\w: ]+\] Extracted .*?\n//sg	if($self->{suppress_extracted});
 		  $report .=
 			"\nThis report was machine-generated by CPAN::YACSmoke $VERSION.\n";
 		  return $report;
@@ -207,10 +274,11 @@ my $extn = qr/(?:\.(?:tar\.gz|tgz|zip))/;	# supported archive extensions
 
 		  return unless ($self->{cpantest});
 
-                  # Simplified algorithm for reporting: don't send a report
-                  # if we get the same results as the last report sent, or
-                  # if we it passed last test but not now, or it passed now
-                  # but not the last test.
+          # Simplified algorithm for reporting: 
+          # * don't send a report if
+          #   - we get the same results as the last report sent
+          #   - it passed the last test but not now
+          #   - it didn't pass the last test or now
 
 		  return if (defined $checked && (
                     ($checked eq $grade)                     ||
@@ -242,13 +310,14 @@ my @CPANPLUS_FIELDS = qw(
         prefer_bin prefer_makefile
         makeflags makemakerflags
         md5 signature
+        extractdir fetchdir
 );
 
 my @CONFIG_FIELDS = (@CPANPLUS_FIELDS, qw(
 	recent_list_age ignore_cpanplus_bugs fail_max
 	exclude_dists test_max audit_log
 	ignore_bad_prereqs report_pass_only
-	allow_retries flush_flag
+	allow_retries flush_flag suppress_extracted
 ));
 
 
@@ -274,6 +343,8 @@ otherwise noted) are:
   makemakerflags
   md5
   signature
+  extractdir
+  fetchdir
 
 CPAN::YACSmoke configuration settings are:
 
@@ -283,6 +354,7 @@ CPAN::YACSmoke configuration settings are:
   exclude_dists
   test_max
   allow_retries
+  suppress_extracted
   flush_flag         - used by purge()
 
   list_from          - List plugin required, default Recent
@@ -353,7 +425,7 @@ sub new {
 
 	## set internal defaults
 	my $self  = {
-                conf                 => $conf,
+        conf                 => $conf,
 		checked              => undef,
 		ignore_cpanplus_bugs => ($CPANPLUS::Backend::VERSION >= 0.052),
 		fail_max             => 3,     # max failed versions to try
@@ -375,23 +447,24 @@ sub new {
 	$self->{skiptest} = 0;
 	$self->{prereqs}  = 2; # force to ask callback
 
-        # Makefile.PL shows which tests failed, whereas Build.PL does
-        # not when reports are sent through CPANPLUS 0.053, hence the
-        # prefer_makefile=1 default.
+    # Makefile.PL shows which tests failed, whereas Build.PL does
+    # not when reports are sent through CPANPLUS 0.053, hence the
+    # prefer_makefile=1 default.
 
-        $self->{prefer_makefile} = 1;
+    $self->{prefer_makefile} = 1;
 
-        # If we have TEST_VERBOSE=1 by default, then many FAIL reports
-        # will be huge. A lot of module authors will want that, but
-        # it's not the best idea to send those out immediately.
+    # If we have TEST_VERBOSE=1 by default, then many FAIL reports
+    # will be huge. A lot of module authors will want that, but
+    # it's not the best idea to send those out immediately.
 
-        ## $self->{makeflags} = 'TEST_VERBOSE=1';
+    ## $self->{makeflags} = 'TEST_VERBOSE=1';
 	
 	my %config = @_;
 
 	## config_file is an .ini file
 
-	$config{config_file} ||= catfile($self->basedir(), CONFIG_FILE);
+	$config{config_file} ||=
+          file($self->basedir(), CONFIG_FILE)->stringify;
 
 	if($config{config_file} && -r $config{config_file}) {
 		my $cfg = Config::IniFiles->new(-file => $config{config_file});
@@ -406,9 +479,9 @@ sub new {
 	}
 
 	if ($self->{audit_log}) {
-	  my ($vol, $path, $file) = splitpath $self->{audit_log};
+	  my ($vol, $path, $file) = splitpath($self->{audit_log});
 	  unless ($vol || $path) {
-	    $self->{audit_log} = catfile($self->basedir(), $file);
+	    $self->{audit_log} = file($self->basedir(), $file)->stringify;
 	  }
 	}
 
@@ -446,7 +519,8 @@ sub new {
 
 	## determine the database file
 
-	$self->{database_file} ||= catfile($self->basedir(), DATABASE_FILE);
+	$self->{database_file} ||=
+          file($self->basedir(), DATABASE_FILE)->stringify;
 
 	$self->_connect_db();
 	$self->_connect_cpanplus($conf);
@@ -475,20 +549,14 @@ Obtains the users home directory
 
 sub homedir {
   my $self = shift;
-  return $self->{homedir} = shift	if (@_);
+  return $self->{homedir} = dir(shift)	if (@_);
+
+  my $home = dir(home());
+
+  $self->{homedir} = $home;
 
-  unless (defined $self->{homedir}) {
-    if ($^O eq "MSWin32") { # bug in File::HomeDir <= 0.06
-      $self->{homedir} = $ENV{HOME}      ||
-	($ENV{HOMEDRIVE}.$ENV{HOMEPATH}) ||
-	  $ENV{USERPROFILE}              ||
-	    home();
-    } else {
-      $self->{homedir} = home();
-    }
-  }
   $self->_audit("homedir = " . $self->{homedir});
-  return $self->{homedir};
+  return $self->{homedir}->stringify;
 }
 
 =item basedir
@@ -520,11 +588,11 @@ sub builddir {
 
   require Config;
 
-  return File::Spec->catdir(
+  return dir(
     $self->{conf}->get_conf('base'),
 	$Config::Config{version},
 	$self->{conf}->_get_build('moddir'),
-  );
+  )->stringify;
 }
 
 
@@ -648,7 +716,7 @@ sub test {
 
   my @distros = @_;
   unless (@distros) {
-    @distros = $smoker->{plugin}->download_list(1);
+    @distros = $smoker->{plugin}->download_list();
     unless (@distros) {
       exit error("No new distributions uploaded to be tested");
     }
@@ -687,12 +755,9 @@ sub test {
 
       my $grade = $smoker->{checked}->{$distver} || 'ungraded';
 
-      if ((!defined $grade) ||
+      if (($grade eq 'ungraded') ||
 	  ($smoker->{allow_retries} && $grade =~ /$smoker->{allow_retries}/)) {
 
-	    # We no longer re-test 'unknown' since that rating is reserved
-	    # for modules with no tests.
-
 	    my $mod = $smoker->{cpan}->parse_module( module => $distpathver)
 	      or error("Invalid distribution $distver\n");
 
@@ -719,7 +784,9 @@ sub test {
 
 			# TODO: check the $stat and react appropriately
 
-			$smoker->_audit(CPANPLUS::Error->stack_as_string());
+            my $stack = CPANPLUS::Error->stack_as_string();
+            $stack =~ s/\[MSG\] \[[\w: ]+\] Extracted .*?\n//sg	if($smoker->{suppress_extracted});
+            $smoker->_audit($stack);
 
 			# TODO: option to mark uncompleted tests as aborted vs ungraded
 			#       aborted should indicate a fault in testing the distribution
@@ -936,8 +1003,9 @@ sub purge {
 							"'] has been purged");
 			delete $smoker->{checked}->{"$dist-$vers"};
 			if($flush) {
-				my $builddir = catfile($smoker->basedir(), "$dist-$vers");
-				rmtree($builddir)	if(-d $builddir);
+		          my $builddir =
+                            file($smoker->basedir(), "$dist-$vers")->stringify;
+			  rmtree($builddir)	if(-d $builddir);
 			}
 		}
 		elsif($smoker->{checked}->{"$dist-$vers"} eq 'pass') {
@@ -1070,6 +1138,19 @@ previously.
 
 There are no pass through configuration settings.
 
+=head2 PlainTextList
+
+The PlainTextList plugin allows for a locally created plain text file,
+listing all the distributions to be tested in a single run. Note that the
+excluded list and text_max settings will still apply.
+
+Pass through configuration settings:
+
+  %config = {
+	  list_from => 'PlainTextList',
+ 	  data => $my_data_file,
+  };
+
 =head2 Writing A Plugin
 
 For an example, see one of the above plugins, or one of the known plugin
@@ -1112,6 +1193,8 @@ CPAN. Uses KobeSearch by default, but another similar page can be requested.
 
 =back
 
+=for readme continue
+
 =head1 CAVEATS
 
 This is a proto-type release. Use with caution and supervision.
